package org.tupeloproject.pchal2;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.tupeloproject.kernel.Context;
import org.tupeloproject.kernel.OperatorException;
import org.tupeloproject.kernel.Transformer;
import org.tupeloproject.kernel.TripleMatcher;
import org.tupeloproject.kernel.impl.MemoryContext;
import org.tupeloproject.rdf.Resource;
import org.tupeloproject.rdf.Triple;
import static org.tupeloproject.rdf.Vocabulary.Owl;
import org.tupeloproject.rdf.query.Pattern;
import org.tupeloproject.rdf.query.TransitiveClosure;
import org.tupeloproject.util.Iterators;

/**
 * Owlet does some very basic OWL processing
 */
public class Owlet {
    /**
     * Processes an entire context merging sameAs
     * @param context
     * @throws OperatorException
     */
    public static Set<Triple> sameAs(Context context) throws OperatorException {
        // fake symmetry
        Context c = new MemoryContext(context.getTriples());
        Transformer xf = new Transformer();
        xf.addInPattern("a",Owl.SAME_AS,"b");
        xf.addOutPattern("b",Owl.SAME_AS,"a");
        c.perform(xf);
        c.addTriples(xf.getResult());
        // get all owl:sameAs assertions
        TripleMatcher tm = new TripleMatcher();
        tm.setPattern(new Pattern("a",Owl.SAME_AS,"b"));
        c.perform(tm);
        // each transitive closure of owl:sameAs defines an equivalence class;
        // we'll pick one resource from each equivalence class as the canonical one,
        // just by sorting them by their natural order
        // map each resource to the canonical resource in the equivalence class
        // to which it belongs
        final Map<Resource,Resource> eClasses = new HashMap<Resource,Resource>();
        // iterate over the owl:sameAs assertions
        for(Triple t : tm.getResult()) {
            Resource s = t.getSubject();
            Resource o = t.getObject();
            // skip if either term is already in an equivalence class
            if(eClasses.get(s) == null && eClasses.get(o) == null) {
                // compute the transitive closure of owl:sameAs
                // rooted at this subject
                TransitiveClosure tc = new TransitiveClosure();
                tc.setPredicate(Owl.SAME_AS);
                tc.setSeed(s);
                TreeSet<Resource> sames = new TreeSet<Resource>();
                sames.add(s);
                sames.addAll(tc.close(c));
                // first one in the set is the "canonical" one for the purposes
                // of this operation
                Resource canonical = sames.first();
                // record this as the equivalence class
                boolean first = true;
                for(Resource same : sames) {
                    if(!first) {
                        eClasses.put(same,canonical);
                    }
                    first = false;
                }
                // FIXME debug
//                System.out.printf("[%s",canonical);
//                first = true;
//                for(Resource same : sames) {
//                    if(!first) {
//                        System.out.printf(" %s",same);
//                    }
//                    first = false;
//                }
//                System.out.printf("]\n");
            }
        }
        // dump the results
        final Iterator<Triple> tit = c.iterator();
        return Iterators.asSet(new Iterator<Triple>() {
            int nMods = 0;
            public boolean hasNext() {
                if(tit.hasNext()) {
                    return true;
                } else {
                    System.out.printf("%d term(s) canonicalized\n",nMods); // FIXME debug
                    return false;
                }
            }
            public Triple next() {
                Triple original = tit.next();
                if(original.getPredicate().equals(Owl.SAME_AS)) {
                    return original;
                } else {
                    Triple modified = new Triple();
                    for(int i = 0; i < 3; i++) {
                        Resource term = original.get(i);
                        Resource mod = eClasses.get(term);
                        if(mod != null && !mod.equals(term)) {
                            nMods++;
                            modified.set(i,mod);
                        } else {
                            modified.set(i,term);
                        }
                    }
                    return modified;
                }
            }
            public void remove() {
                throw new UnsupportedOperationException();
            }
        });
    }
}