package org.tupeloproject.pchal2;

import java.io.IOException;
import java.net.URI;
import java.util.Set;

import org.tupeloproject.kernel.Context;
import org.tupeloproject.kernel.OperatorException;
import org.tupeloproject.kernel.Transformer;
import org.tupeloproject.kernel.impl.MemoryContext;
import org.tupeloproject.rdf.Resource;
import org.tupeloproject.rdf.Triple;
import static org.tupeloproject.rdf.Vocabulary.Rdf;
import static org.tupeloproject.rdf.Vocabulary.Rdfs;

/**
 * SdgTest
 */
public class MindswapTest extends TransformerStrategy {
    static Resource prov(String sfx) {
        return Resource.uriRef("http://provenance.mindswap.org/provenance.owl#"+sfx);
    }

    void xform(Context c, String service, String[] inputs, String[] outputs) throws OperatorException {
        for(String input : inputs) {
            Transformer t = new Transformer();
            t.addInPattern("step",prov("serviceUsed"),prov(service));
            t.addInPattern("step",prov(input),"datum");
            t.addOutPattern("datum",Rdf.TYPE,Ontology.DATASET_TYPE);
            t.addOutPattern("step",Ontology.STEP_HAS_INPUT,"datum");
            xform(c,t);
        }
        for(String output : outputs) {
            Transformer t = new Transformer();
            t.addInPattern("step",prov("serviceUsed"),prov(service));
            t.addInPattern("step",prov(output),"datum");
            t.addOutPattern("datum",Rdf.TYPE,Ontology.DATASET_TYPE);
            t.addOutPattern("step",Ontology.STEP_HAS_OUTPUT,"datum");
            xform(c,t);
        }
    }

    public Set<Triple> getTriples(String path) throws OperatorException, IOException {
        String frag = path.replaceFirst("/data/MINDSWAP2/","");
        String baseUri = "http://provenance.mindswap.org/"+frag;
        MemoryContext c = new MemoryContext(super.getTriples(path,URI.create(baseUri)));
        // process steps
        Transformer t = new Transformer();
        t.addInPattern("step", Rdf.TYPE, prov("ServiceExecution"));
        t.addOutPattern("step", Rdf.TYPE, Ontology.STEP_TYPE);
        xform(c,t);
        // align-warp input images
        for(String awInput : new String[] { "hasInputImage", "hasInputHeader" }) {
            t = new Transformer();
            t.addInPattern("step", prov("serviceUsed"), prov("align_warp"));
            t.addInPattern("step", prov(awInput), "datum");
            t.addOutPattern("datum", Rdf.TYPE, Ontology.DATASET_TYPE);
            t.addOutPattern("datum", Rdfs.LABEL, "datum");
            t.addOutPattern("step", Ontology.STEP_HAS_INPUT, "datum");
            xform(c,t);
        }
        // align-warp outputs
        t = new Transformer();
        t.addInPattern("step",prov("hasOutputParameters"),"datum");
        t.addOutPattern("datum",Rdf.TYPE,Ontology.DATASET_TYPE);
        t.addOutPattern("step",Ontology.STEP_HAS_OUTPUT,"datum");
        xform(c,t);
        // reslice
        xform(c, "reslice",
              new String[] { "hasInputParameters" }, // inputs
              new String[] { "hasOutputImage", "hasOutputHeader" }); // outputs
        // softmean
        xform(c, "softmean",
              new String[] { "hasInputImage", "hasInputHeader" },
              new String[] { "hasOutputHeader", "hasOutputImage" });
        // slicer
        xform(c, "slicer",
              new String[] { "hasInputImage", "hasInputHeader" },
              new String[] { "hasOutputSlice" });
        // convert
        xform(c, "convert",
              new String[] { "hasInputSlice" },
              new String[] { "hasOutputGraphic" });
        // double process
        xform(c, "pgmtoppm",
              new String[] { "hasInputSlice" },
              new String[] { "hasOutputGraphic" });
        //
        return c.getTriples();
    }

    protected String[] getFiles(int i) {
        String pfx = "/data/MINDSWAP2/";
        String sfx = ".owl";
        String files[] = null;
        switch(i) {
            case 1:
                files = new String[] {
                        "Align_WarpExecution-1155932184.50005",
                        "Align_WarpExecution-1155932184.52346",
                        "Align_WarpExecution-1155932184.54673",
                        "Align_WarpExecution-1155932184.56954",
                        "ResliceExecution-1155932184.59296",
                        "ResliceExecution-1155932184.59735",
                        "ResliceExecution-1155932184.60179",
                        "ResliceExecution-1155932184.60617",
                };
                break;
            case 2:
                files = new String[] {
                        "SoftmeanExecution-1155932184.61051",
                };
                break;
            case 3:
                files = new String[] {
                        "ConvertExecution-1155932184.62982",
                        "ConvertExecution-1155932184.63421",
                        "ConvertExecution-1155932184.63854",
                        "SlicerExecution-1155932184.61546",
                        "SlicerExecution-1155932184.6202",
                        "SlicerExecution-1155932184.62498",
                };
                break;
        }
//        files = new String[] {
//                "Align_WarpExecution-1157476801.77876",
//                "Align_WarpExecution-1157476801.81584",
//                "Align_WarpExecution-1157476801.83962",
//                "Align_WarpExecution-1157476801.86305",
//                "DoubleProcessExecution-1157476801.92867",
//                "DoubleProcessExecution-1157476801.93301",
//                "DoubleProcessExecution-1157476801.93765",
//                "ResliceExecution-1157476801.88659",
//                "ResliceExecution-1157476801.89519",
//                "ResliceExecution-1157476801.8996",
//                "ResliceExecution-1157476801.90402",
//                "SlicerExecution-1157476801.91373",
//                "SlicerExecution-1157476801.91893",
//                "SlicerExecution-1157476801.92382",
//                "SoftmeanExecution-1157476801.9084",
//        };
        //pfx+"Q7_Part1"+sfx,
        //pfx+"Q7_Part2"+sfx,
        //pfx+"Q7_Part3"+sfx
        return getFiles(pfx,sfx,files);
    }

    protected Resource getAtlasXGraphic() {
        return Resource.uriRef("http://provenance.mindswap.org/files/ConvertExecution-1155932184.62982.owl#Graphic1155932184.62982");
    }
}
