package org.tupeloproject.pchal2;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;

import org.tupeloproject.kernel.Context;
import org.tupeloproject.kernel.impl.MemoryContext;
import org.tupeloproject.rdf.Triple;
import org.tupeloproject.rdf.ntriples.NTriplesInputStream;

import org.junit.Test;

/**
 * MultiStrategy
 */
public class MultiStrategyTest {
    Map<String, Set<Triple>> cache = new HashMap<String, Set<Triple>>();
    Set<Triple> sames;

    Set<Triple> getTriples(Strategy s, int n) throws Exception {
        String[] files = s.getFiles(n);
        String key = "";
        for(String k : files) {
            key += k + " ";
        }
        Set<Triple> cached = cache.get(key);
        if(cached == null) {
            cached = new HashSet<Triple>();
            cached.addAll(s.infer(s.getAllTriples(files)));
            cache.put(key, cached);
        }
        return cached;
    }

    public Set<Triple> getSames() {
        if(sames == null) {
            NTriplesInputStream ntis = new NTriplesInputStream
                    (getClass().getResourceAsStream("/nt/sames.nt"));
            Set<Triple> s = new HashSet<Triple>();
            for(Triple t : ntis) {
                s.add(t);
            }
            System.out.printf("read %d sameAs assertions from sames.nt\n", s.size());
            sames = s;
        }
        return sames;
    }

    @Test
    public void testAll() throws Exception {
        Set<Class> strategies = new HashSet<Class>();
        strategies.add(CesnetTest.class);
        strategies.add(Es3Test.class);
        strategies.add(KarmaTest.class);
        strategies.add(MindswapTest.class);
        strategies.add(MyGridTest.class);
        strategies.add(SdgTest.class);
        strategies.add(VisTrailsTest.class);
        Class[] cs = (Class[]) strategies.toArray(new Class[] {});
        Strategy s[] = new Strategy[3];
        for(int i = 0; i < cs.length; i++) {
            s[0] = (Strategy) cs[i].newInstance();
            for(int j = 0; j < cs.length; j++) {
                s[1] = (Strategy) cs[j].newInstance();
                for(int k = 0; k < cs.length; k++) {
                    s[2] = (Strategy) cs[k].newInstance();
                    long then = System.currentTimeMillis();
                    Context c = new MemoryContext();
                    c.addTriples(getTriples(s[0], 1));
                    c.addTriples(getTriples(s[1], 2));
                    c.addTriples(getTriples(s[2], 3));
                    c.addTriples(getSames());
                    String pattern = "org.tupeloproject.pchal2.(.*)Test";
                    String fn = String.format
                            ("%s_%s_%s",
                             cs[i].getName().replaceAll(pattern, "$1"),
                             cs[j].getName().replaceAll(pattern, "$1"),
                             cs[k].getName().replaceAll(pattern, "$1"));
                    Set<Triple> canonical = new HashSet<Triple>();
                    System.out.print("stitching... ");
                    for(Triple t : Owlet.sameAs(c)) {
                        canonical.add(t);
                    }
                    Strategy.graph(fn, canonical);
                    // now run query 1
                    SortedSet<String> queryOneResult = Strategy.runQueryOne(c.getTriples(),s[2].getAtlasXGraphic());
                    boolean good = true;
                    if(queryOneResult.size() != 11) { good = false; }
                    Iterator<String> it = queryOneResult.iterator();
                    if(!it.next().equals(" 1. align_warp")) { good = false; }
                    if(!it.next().equals(" 2. align_warp")) { good = false; }
                    if(!it.next().equals(" 3. align_warp")) { good = false; }
                    if(!it.next().equals(" 4. align_warp")) { good = false; }
                    if(!it.next().equals(" 5. reslice")) { good = false; }
                    if(!it.next().equals(" 6. reslice")) { good = false; }
                    if(!it.next().equals(" 7. reslice")) { good = false; }
                    if(!it.next().equals(" 8. reslice")) { good = false; }
                    if(!it.next().equals(" 9. softmean")) { good = false; }
                    if(!it.next().equals("10. slicer")) { good = false; }
                    if(!it.next().equals("13. convert")) { good = false; }
                    System.out.printf("QUERY 1> %s %s\n",fn,good?"1":"0");
                    long now = System.currentTimeMillis();
                    long elapsed = now - then;
                    System.out.printf("did %s in %d ms\n",fn,elapsed);
                }
            }
        }
    }
}
