package org.tupeloproject.pchal2;

import java.io.IOException;
import java.util.Set;

import org.tupeloproject.kernel.Context;
import org.tupeloproject.kernel.OperatorException;
import org.tupeloproject.kernel.Transformer;
import org.tupeloproject.kernel.impl.MemoryContext;
import org.tupeloproject.rdf.Resource;
import org.tupeloproject.rdf.Triple;
import static org.tupeloproject.rdf.Vocabulary.Owl;
import static org.tupeloproject.rdf.Vocabulary.Rdf;
import static org.tupeloproject.rdf.Vocabulary.Rdfs;

/**
 * MyGridTest
 */
public class MyGridTest extends TransformerStrategy {
    static Resource myGrid(String sfx) {
        return Resource.uriRef("http://www.mygrid.org.uk/provenance#"+sfx);
    }

    public Set<Triple> getTriples(String path) throws OperatorException, IOException {
        MemoryContext c = new MemoryContext(super.getTriples(path));
        // process steps
        Transformer t = new Transformer();
        t.addInPattern("step", Rdf.TYPE, myGrid("ProcessRun"));
        t.addInPattern("step", myGrid("runsProcess"), "label");
        t.addOutPattern("step", Rdf.TYPE, Ontology.STEP_TYPE);
        t.addOutPattern("step", Rdfs.LABEL, "label");
        xform(c,t);
        // process output
        t = new Transformer();
        t.addInPattern("step", Rdf.TYPE, myGrid("ProcessRun"));
        t.addInPattern("step", myGrid("processOutput"), "output");
        t.addOutPattern("step", Ontology.STEP_HAS_OUTPUT, "output");
        xform(c,t);
        // process input
        t = new Transformer();
        t.addInPattern("step", Rdf.TYPE, myGrid("ProcessRun"));
        t.addInPattern("step", myGrid("processInput"), "input");
        t.addOutPattern("step", Ontology.STEP_HAS_INPUT, "input");
        xform(c,t);
        // datasets
        t = new Transformer();
        t.addInPattern("datum", Rdf.TYPE, myGrid("AtomicData"));
        t.addInPattern("datum", myGrid("outputDataHasName"), "name");
        t.addOutPattern("datum", Rdf.TYPE, Ontology.DATASET_TYPE);
        t.addOutPattern("datum", Rdfs.LABEL, "name");
        xform(c,t);
        return c.getTriples();
    }

    public Set<Triple> infer(Set<Triple> groundTruth) throws OperatorException {
        Context c = new MemoryContext(groundTruth);
        // every pair of datasets with the same label is the same
        Transformer t = new Transformer();
        t.addInPattern("datum1", Rdf.TYPE, Ontology.DATASET_TYPE);
        t.addInPattern("datum2", Rdf.TYPE, Ontology.DATASET_TYPE);
        t.addInPattern("datum1", Rdfs.LABEL, "label");
        t.addInPattern("datum2", Rdfs.LABEL, "label");
        t.addOutPattern("datum1", Owl.SAME_AS, "datum2");
        xform(c,t);
        // every pair of steps with the same label is the same
        t = new Transformer();
        t.addInPattern("step1", Rdf.TYPE, Ontology.STEP_TYPE);
        t.addInPattern("step2", Rdf.TYPE, Ontology.STEP_TYPE);
        t.addInPattern("step1", Rdfs.LABEL, "label");
        t.addInPattern("step2", Rdfs.LABEL, "label");
        t.addOutPattern("step1", Owl.SAME_AS, "step2");
        xform(c,t);
        //
        return Owlet.sameAs(c);
    }

    protected String[] getFiles(int i) {
        String pfx = "/data/MyGrid2/SecondChallenge_myGrid_";
        String sfx = ".rdf";
        //pfx+"Q7_Part1"+sfx,
        //pfx+"Q7_Part2"+sfx,
        //pfx+"Q7_Part3"+sfx
        String[] files = getFiles(pfx,sfx,"Part1","Part2","Part3");
        return new String[] { files[i-1] };
    }

    protected Resource getAtlasXGraphic() {
        return Resource.uriRef("urn:lsid:net.sf.taverna:dataItem:406ccd75-8271-443b-b3b7-9803b68e316e");
    }
}
