package org.tupeloproject.pchal2;

import java.io.IOException;
import java.util.Set;

import org.tupeloproject.kernel.OperatorException;
import org.tupeloproject.kernel.Transformer;
import org.tupeloproject.kernel.impl.MemoryContext;
import org.tupeloproject.rdf.Resource;
import org.tupeloproject.rdf.Triple;
import static org.tupeloproject.rdf.Vocabulary.Dc;
import static org.tupeloproject.rdf.Vocabulary.Rdf;
import static org.tupeloproject.rdf.Vocabulary.Rdfs;

/**
 * SdgTest
 */
public class SdgTest extends TransformerStrategy {
    static Resource sdg(String sfx) {
        return Resource.uriRef("http://sdg.pnl.gov/provenance#"+sfx);
    }

    public Set<Triple> getTriples(String path) throws OperatorException, IOException {
        MemoryContext c = new MemoryContext(super.getTriples(path));
        // process steps
        Transformer t = new Transformer();
        t.addInPattern("step", Rdf.TYPE, sdg("ActorInstance"));
        t.addInPattern("step", Dc.TITLE, "label");
        t.addOutPattern("step", Rdf.TYPE, Ontology.STEP_TYPE);
        t.addOutPattern("step", Rdfs.LABEL, "label");
        xform(c,t);
        // datasets
        t = new Transformer();
        t.addInPattern("datum", Rdf.TYPE, sdg("DataInstance"));
        t.addInPattern("datum", Dc.TITLE, "label");
        t.addOutPattern("datum", Rdf.TYPE, Ontology.DATASET_TYPE);
        t.addOutPattern("datum", Rdfs.LABEL, "label");
        xform(c,t);
        // steps' output
        t = new Transformer();
        t.addInPattern("step", Rdf.TYPE, sdg("ActorInstance"));
        t.addInPattern("step", sdg("hasOutput"), "datum");
        t.addOutPattern("step", Ontology.STEP_HAS_OUTPUT, "datum");
        xform(c,t);
        // datasets use as input
        t = new Transformer();
        t.addInPattern("datum", Rdf.TYPE, sdg("DataInstance"));
        t.addInPattern("datum", sdg("isInput"), "step");
        t.addOutPattern("step", Ontology.STEP_HAS_INPUT, "datum");
        xform(c,t);
        return c.getTriples();
    }

    protected String[] getFiles(int i) {
        String pfx = "/data/SDG2/";
        String sfx = "-final.xml";
        //pfx+"Q7_Part1"+sfx,
        //pfx+"Q7_Part2"+sfx,
        //pfx+"Q7_Part3"+sfx
        String[] files = getFiles(pfx,sfx,"stage1","stage2","stage3");
        return new String[] { files[i-1] }; 
    }

    protected Resource getAtlasXGraphic() {
        return Resource.uriRef("http://penrose.pnl.gov:8080/alfresco/webdav/workflow-2108762597/1279676452atlas-x.gif");
    }
}
