package org.tupeloproject.pchal2;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.Set;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.URIResolver;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.tupeloproject.rdf.Triple;
import org.tupeloproject.rdf.ntriples.NTriplesInputStream;
import org.tupeloproject.util.Iterators;
import org.tupeloproject.util.Xml;

/**
 * XslStrategy is for converting XML provenance data to serialized RDF using an
 * XSLT stylesheet.
 */
public abstract class XslStrategy extends Strategy {
    final String BASE_URI = "/xsl/";

    protected abstract String getStylesheet();

    public static Source getSource(String resource) {
        return new StreamSource(XslStrategy.class.getResourceAsStream(resource));
    }

    public void rawTransform(String xml, String xsl, OutputStream os) throws TransformerException {
        Source xmlSource = getSource(xml);
        Result result = new StreamResult(os);
        URIResolver resolver = new URIResolver() {
            public Source resolve(String href, String theBaseUri) {
                return getSource(BASE_URI+href);
            }
        };
        Xml.transform(xmlSource, resolver.resolve(xsl,BASE_URI), result, resolver);
    }

    public Set<Triple> transform(String xml, String xsl) throws TransformerException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        rawTransform(xml,xsl,baos);
        ByteArrayInputStream nt = new ByteArrayInputStream(baos.toByteArray());
        NTriplesInputStream ntis = new NTriplesInputStream(nt);
        return Iterators.asSet(ntis.iterator());
    }

    public Set<Triple> getTriples(String resourcePath) throws Exception {
        return transform(resourcePath,getStylesheet());
    }
}
