<?xml version="1.0"?>
<!DOCTYPE stylesheet [
<!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#">
<!ENTITY t2 "tag:tupeloproject.org,2006:/2.0/pc2/">
<!ENTITY rdfs "http://www.w3.org/2000/01/rdf-schema#">
]>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:include href="rdf-ntriples.xsl"/>

  <xsl:template match="/ES3response/link">
    <xsl:apply-templates select="*/object"/>
    <xsl:apply-templates select="to"/>
  </xsl:template>

  <!-- inputs -->
  <xsl:template match="to[object/@type='transformation'][preceding-sibling::from[object/@type='file']]">
    <xsl:call-template name="triple">
      <xsl:with-param name="s">&lt;urn:uuid:<xsl:value-of select="object/uuid"/>&gt;</xsl:with-param>
      <xsl:with-param name="p">&t2;stepHasInput</xsl:with-param>
      <xsl:with-param name="o">&lt;&t2;md5sum/<xsl:value-of select="preceding-sibling::from/object/md5sum"/>&gt;</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates select="object"/>
  </xsl:template>

  <!-- outputs -->
  <xsl:template match="to[object/@type='file'][preceding-sibling::from[object/@type='transformation']]">
    <xsl:call-template name="triple">
      <xsl:with-param name="s">&lt;urn:uuid:<xsl:value-of select="preceding-sibling::from/object/uuid"/>&gt;</xsl:with-param>
      <xsl:with-param name="p">&t2;stepHasOutput</xsl:with-param>
      <xsl:with-param name="o">&lt;&t2;md5sum/<xsl:value-of select="object/md5sum"/>&gt;</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates select="object"/>
  </xsl:template>

  <xsl:template match="object[@type='transformation']">
    <!-- only do this once per unique transformation uuid -->
    <xsl:if test="not(uuid=../../preceding::*/object/uuid)">
      <!-- step rdf:type t2:Step -->
      <xsl:call-template name="triple">
        <xsl:with-param name="s">&lt;urn:uuid:<xsl:value-of select="uuid"/>&gt;</xsl:with-param>
        <xsl:with-param name="p">&rdf;type</xsl:with-param>
        <xsl:with-param name="o">&lt;&t2;Step&gt;</xsl:with-param>
      </xsl:call-template>
      <!-- step rdfs:label "{mnemonic label}" -->
      <xsl:call-template name="triple">
        <xsl:with-param name="s">&lt;urn:uuid:<xsl:value-of select="uuid"/>&gt;</xsl:with-param>
        <xsl:with-param name="p">&rdfs;label</xsl:with-param>
        <xsl:with-param name="o">&quot;<xsl:value-of select="name"/>&quot;</xsl:with-param>
      </xsl:call-template>
      <!-- step t2:stepExecutedAt "{timestamp}" -->
      <xsl:call-template name="triple">
        <xsl:with-param name="s">&lt;urn:uuid:<xsl:value-of select="uuid"/>&gt;</xsl:with-param>
        <xsl:with-param name="p">&t2;stepExecutedAt</xsl:with-param>
        <xsl:with-param name="o">&quot;<xsl:value-of select="timestamp"/>&quot;</xsl:with-param>
      </xsl:call-template>
      <!-- annotations -->
      <xsl:apply-templates select="name"/>
      <!--xsl:apply-templates select="collection"/-->
    </xsl:if>
  </xsl:template>

  <xsl:template match="object[@type='file']">
    <!-- only do this once per unique file md5sum -->
    <xsl:if test="not(md5sum=../../preceding::*/object/md5sum)">
      <!-- ?file rdf:type t2:Dataset -->
      <xsl:call-template name="triple">
        <xsl:with-param name="s">&lt;&t2;md5sum/<xsl:value-of select="md5sum"/>&gt;</xsl:with-param>
        <xsl:with-param name="p">&rdf;type</xsl:with-param>
        <xsl:with-param name="o">&lt;&t2;Dataset&gt;</xsl:with-param>
      </xsl:call-template>
      <!-- ?file rdfs:label "{localId}" -->
      <xsl:call-template name="triple">
        <xsl:with-param name="s">&lt;&t2;md5sum/<xsl:value-of select="md5sum"/>&gt;</xsl:with-param>
        <xsl:with-param name="p">&rdfs;label</xsl:with-param>
        <xsl:with-param name="o">&quot;<xsl:value-of select="localId"/>&quot;</xsl:with-param>
      </xsl:call-template>
      <!-- annotations -->
      <xsl:apply-templates select="localId"/>
      <!--xsl:apply-templates select="collection"/-->
    </xsl:if>
  </xsl:template>

  <!-- suppress all other processing of "to" elements -->
  <xsl:template match="to"/>

  <!-- these templates gets applied to specific sub-elements of object -->
  <xsl:template match="object[@type='transformation']/*">
    <xsl:call-template name="annotation">
      <xsl:with-param name="target">&lt;urn:uuid:<xsl:value-of select="../uuid"/>&gt;</xsl:with-param>
      <xsl:with-param name="id">_:<xsl:value-of select="generate-id(.)"/></xsl:with-param>
      <xsl:with-param name="name"><xsl:value-of select="local-name()"/></xsl:with-param>
      <xsl:with-param name="value"><xsl:value-of select="."/></xsl:with-param>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="object[type='file']/*">
    <xsl:call-template name="annotation">
      <xsl:with-param name="target">&lt;&t2;md5sum/<xsl:value-of select="../md5sum"/>&gt;</xsl:with-param>
      <xsl:with-param name="id">_:<xsl:value-of select="generate-id(.)"/></xsl:with-param>
      <xsl:with-param name="name"><xsl:value-of select="local-name()"/></xsl:with-param>
      <xsl:with-param name="value"><xsl:value-of select="."/></xsl:with-param>
    </xsl:call-template>
  </xsl:template>

</xsl:stylesheet>