<?xml version="1.0"?>
<!DOCTYPE stylesheet [
<!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#">
<!ENTITY t2 "tag:tupeloproject.org,2006:/2.0/pc2/">
<!ENTITY rdfs "http://www.w3.org/2000/01/rdf-schema#">
]>
<xsl:stylesheet version="1.0"
                xmlns:wor="http://lead.extreme.indiana.edu/namespaces/2006/06/workflow_tracking"
                xmlns:S="http://schemas.xmlsoap.org/soap/envelope/"
                xmlns:alignwarpservice_runtypens="http://www.extreme.indiana.edu/karma/challenge2/AlignWarpService/xsd"
                xmlns:resliceservice_runtypens="http://www.extreme.indiana.edu/karma/challenge2/ResliceService/xsd"
                xmlns:softmeanservice_runtypens="http://www.extreme.indiana.edu/karma/challenge2/SoftmeanService/xsd"
                xmlns:slicerservice_runtypens="http://www.extreme.indiana.edu/karma/challenge2/SlicerService/xsd"
                xmlns:convertservice_runtypens="http://www.extreme.indiana.edu/karma/challenge2/ConvertService/xsd"
                xmlns:pgmtoppmservice_runtypens="http://www.extreme.indiana.edu/karma/challenge2/PgmToPpmService/xsd"
                xmlns:pnmtojpegservice_runtypens="http://www.extreme.indiana.edu/karma/challenge2/PnmToJpegService/xsd"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:include href="rdf-ntriples.xsl"/>

  <xsl:template match="/activityDump">
    <xsl:apply-templates/>
  </xsl:template>

  <!-- step execution including input params is a wor:serviceInvoked -->
  <xsl:template match="wor:serviceInvoked">
    <!-- wor:notificationSource/@wor:workflowNodeID identifies the step, but it's unqualified -->
    <!-- so we qualify it -->
    <xsl:call-template name="triple">
      <xsl:with-param name="s">&lt;&t2;karma/<xsl:value-of select="wor:notificationSource/@wor:workflowNodeID"/>&gt;</xsl:with-param>
      <xsl:with-param name="p">&rdf;type</xsl:with-param>
      <xsl:with-param name="o">&lt;&t2;Step&gt;</xsl:with-param>
    </xsl:call-template>
    <!-- use the service id as a mnemonic -->
    <xsl:call-template name="triple">
      <xsl:with-param name="s">&lt;&t2;karma/<xsl:value-of select="wor:notificationSource/@wor:workflowNodeID"/>&gt;</xsl:with-param>
      <xsl:with-param name="p">&rdfs;label</xsl:with-param>
      <xsl:with-param name="o">&quot;<xsl:value-of select="wor:notificationSource/@wor:workflowNodeID"/>&quot;</xsl:with-param>
    </xsl:call-template>
    <!-- descend to service params -->
    <xsl:apply-templates/>
  </xsl:template>

  <!-- step output is a wor:sendingResult -->
  <xsl:template match="wor:sendingResult">
    <xsl:apply-templates/>
  </xsl:template>

  <!-- request bodies contain input, output, and other service params -->
  <xsl:template match="wor:request/wor:body/S:Body">
    <xsl:apply-templates/>
  </xsl:template>

  <!-- this template, when invoked on a service param element, generates a triple whose subject is the step -->
  <!-- also generates ?file rdfs:label "?file" -->
  <xsl:template name="stepParam">
    <xsl:param name="s">&lt;&t2;karma/<xsl:value-of select="../../../../../wor:notificationSource/@wor:workflowNodeID"/>&gt;</xsl:param>
    <xsl:param name="p"/>
    <xsl:param name="o">&lt;<xsl:value-of select="."/>&gt;</xsl:param>
    <xsl:call-template name="triple">
      <xsl:with-param name="s"><xsl:value-of select="$s"/></xsl:with-param>
      <xsl:with-param name="p"><xsl:value-of select="$p"/></xsl:with-param>
      <xsl:with-param name="o"><xsl:value-of select="$o"/></xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="triple">
      <xsl:with-param name="s"><xsl:value-of select="$o"/></xsl:with-param>
      <xsl:with-param name="p">&rdfs;label</xsl:with-param>
      <xsl:with-param name="o">&quot;<xsl:value-of select="."/>&quot;</xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="triple">
      <xsl:with-param name="s"><xsl:value-of select="$o"/></xsl:with-param>
      <xsl:with-param name="p">&rdf;type</xsl:with-param>
      <xsl:with-param name="o">&lt;&t2;Dataset&gt;</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- this template calls stepParam for a param representing the input to a step -->
  <xsl:template name="stepInputParam">
    <xsl:call-template name="stepParam">
      <xsl:with-param name="p">&t2;stepHasInput</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- this template calls stepParam for a param representing the output to a step -->
  <xsl:template name="stepOutputParam">
    <xsl:call-template name="stepParam">
      <xsl:with-param name="p">&t2;stepHasOutput</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- align/warp -->
  <xsl:template match="alignwarpservice_runtypens:Run_InputParams">
    <!-- inputs are identified in the body of several tags -->
    <xsl:for-each select="AnatomyImageFile|AnatomyHeaderFile|ReferenceImageFile|ReferenceHeaderFile">
      <xsl:call-template name="stepInputParam"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="alignwarpservice_runtypens:Run_OutputParams">
    <!-- outputs are identified in the body of several tags -->
    <xsl:for-each select="WarpOutParamFile">
      <xsl:call-template name="stepOutputParam"/>
    </xsl:for-each>
  </xsl:template>

  <!-- reslice -->
  <xsl:template match="resliceservice_runtypens:Run_InputParams">
    <!-- inputs are identified in the body of several tags -->
    <xsl:for-each select="WarpParamFile">
      <xsl:call-template name="stepInputParam"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="resliceservice_runtypens:Run_OutputParams">
    <!-- outputs are identified in the body of several tags -->
    <xsl:for-each select="ResliceOutImageFile|ResliceOutHeaderFile">
      <xsl:call-template name="stepOutputParam"/>
    </xsl:for-each>
  </xsl:template>

  <!-- softmean -->
  <xsl:template match="softmeanservice_runtypens:Run_InputParams">
    <!-- inputs are identified in the body of several tags -->
    <xsl:for-each select="ReslicedImageFile1|ReslicedHeaderFile1|ReslicedImageFile2|ReslicedHeaderFile2|ReslicedImageFile3|ReslicedHeaderFile3|ReslicedImageFile4|ReslicedHeaderFile4">
      <xsl:call-template name="stepInputParam"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="softmeanservice_runtypens:Run_OutputParams">
    <!-- inputs are identified in the body of several tags -->
    <xsl:for-each select="AtlasOutImageFile|AtlasOutHeaderFile">
      <xsl:call-template name="stepOutputParam"/>
    </xsl:for-each>
  </xsl:template>

  <!-- slicer -->
  <xsl:template match="slicerservice_runtypens:Run_InputParams">
    <xsl:for-each select="AtlasImageFile|AtlasHeaderFile">
      <xsl:call-template name="stepInputParam"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="slicerservice_runtypens:Run_OutputParams">
    <xsl:for-each select="SliceOutFile">
      <xsl:call-template name="stepOutputParam"/>
    </xsl:for-each>
  </xsl:template>

  <!-- convert -->
  <xsl:template match="convertservice_runtypens:Run_InputParams">
    <xsl:for-each select="InputImageFile">
      <xsl:call-template name="stepInputParam"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="convertservice_runtypens:Run_OutputParams">
    <xsl:for-each select="ConvertOutImageFile">
      <xsl:call-template name="stepOutputParam"/>
    </xsl:for-each>
  </xsl:template>

  <!-- pgmtoppm -->
  <xsl:template match="pgmtoppmservice_runtypens:Run_InputParams">
    <xsl:for-each select="InputImageFile">
      <xsl:call-template name="stepInputParam"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="pgmtoppmservice_runtypens:Run_OutputParams">
    <xsl:for-each select="PgmToPpmOutImageFile">
      <xsl:call-template name="stepOutputParam"/>
    </xsl:for-each>
  </xsl:template>

  <!-- pnmtojpeg -->
  <xsl:template match="pnmtojpegservice_runtypens:Run_InputParams">
    <xsl:for-each select="InputImageFile">
      <xsl:call-template name="stepInputParam"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="pnmtojpegservice_runtypens:Run_OutputParams">
    <xsl:for-each select="PnmToJpegOutImageFile">
      <xsl:call-template name="stepOutputParam"/>
    </xsl:for-each>
  </xsl:template>
  
</xsl:stylesheet>